@@ -1,5 +1,9 @@
-from sympy.core import (Set, Basic, FiniteSet, EmptySet, Dict, Symbol,
-                        Tuple)
+from __future__ import print_function, division
+
+from sympy.core import S, Basic, Dict, Symbol, Tuple, sympify
+from sympy.core.compatibility import iterable
+from sympy.sets import Set, FiniteSet, EmptySet
+
 
 class Class(Set):
     r"""
@@ -16,6 +20,7 @@ class Class(Set):
     """
     is_proper = False
 
+
 class Object(Symbol):
     """
     The base class for any kind of object in an abstract category.
@@ -25,6 +30,7 @@ class Object(Symbol):
     abstract categories.
     """
 
+
 class Morphism(Basic):
     """
     The base class for any morphism in an abstract category.
@@ -128,6 +134,7 @@ class Morphism(Basic):
         """
         return self.compose(other)
 
+
 class IdentityMorphism(Morphism):
     """
     Represents an identity morphism.
@@ -157,6 +164,7 @@ class IdentityMorphism(Morphism):
     def __new__(cls, domain):
         return Basic.__new__(cls, domain, domain)
 
+
 class NamedMorphism(Morphism):
     """
     Represents a morphism which has a name.
@@ -195,6 +203,7 @@ class NamedMorphism(Morphism):
 
         Examples
         ========
+
         >>> from sympy.categories import Object, NamedMorphism
         >>> A = Object("A")
         >>> B = Object("B")
@@ -205,6 +214,7 @@ class NamedMorphism(Morphism):
         """
         return self.args[2].name
 
+
 class CompositeMorphism(Morphism):
     r"""
     Represents a morphism which is a composition of other morphisms.
@@ -263,13 +273,9 @@ class CompositeMorphism(Morphism):
 
         normalised_components = Tuple()
 
-        # TODO: Fix the unpythonicity.
-        for i in xrange(len(components) - 1):
-            current = components[i]
-            following = components[i + 1]
-
+        for current, following in zip(components, components[1:]):
             if not isinstance(current, Morphism) or \
-                   not isinstance(following, Morphism):
+                    not isinstance(following, Morphism):
                 raise TypeError("All components must be morphisms.")
 
             if current.codomain != following.domain:
@@ -386,6 +392,7 @@ class CompositeMorphism(Morphism):
         """
         return NamedMorphism(self.domain, self.codomain, new_name)
 
+
 class Category(Basic):
     r"""
     An (abstract) category.
@@ -442,7 +449,7 @@ class Category(Basic):
             raise ValueError("A Category cannot have an empty name.")
 
         new_category = Basic.__new__(cls, Symbol(name), Class(objects),
-                                     FiniteSet(commutative_diagrams))
+                                     FiniteSet(*commutative_diagrams))
         return new_category
 
     @property
@@ -509,6 +516,7 @@ class Category(Basic):
         raise NotImplementedError(
             "Obtaining the class of morphisms is not implemented in Category.")
 
+
 class Diagram(Basic):
     r"""
     Represents a diagram in a certain category.
@@ -615,7 +623,7 @@ class Diagram(Basic):
                 Diagram._set_dict_union(morphisms, id_dom, empty)
                 Diagram._set_dict_union(morphisms, id_cod, empty)
 
-            for existing_morphism, existing_props in morphisms.items():
+            for existing_morphism, existing_props in list(morphisms.items()):
                 new_props = existing_props & props
                 if morphism.domain == existing_morphism.codomain:
                     left = morphism * existing_morphism
@@ -697,7 +705,7 @@ class Diagram(Basic):
                 for morphism, props in premises_arg.items():
                     objects |= FiniteSet(morphism.domain, morphism.codomain)
                     Diagram._add_morphism_closure(
-                        premises, morphism, FiniteSet(props))
+                        premises, morphism, FiniteSet(*props) if iterable(props) else FiniteSet(props))
 
         if len(args) >= 2:
             # We also have some conclusions.
@@ -710,15 +718,15 @@ class Diagram(Basic):
 
                 for morphism in conclusions_arg:
                     # Check that no new objects appear in conclusions.
-                    if (morphism.domain in objects) and \
-                       (morphism.codomain in objects):
+                    if ((sympify(objects.contains(morphism.domain)) is S.true) and
+                        (sympify(objects.contains(morphism.codomain)) is S.true)):
                         # No need to add identities and recurse
                         # composites this time.
                         Diagram._add_morphism_closure(
                             conclusions, morphism, empty, add_identities=False,
                             recurse_composites=False)
             elif isinstance(conclusions_arg, dict) or \
-                     isinstance(conclusions_arg, Dict):
+                    isinstance(conclusions_arg, Dict):
                 # The user has supplied a dictionary of morphisms and
                 # their properties.
                 for morphism, props in conclusions_arg.items():
@@ -728,7 +736,7 @@ class Diagram(Basic):
                         # No need to add identities and recurse
                         # composites this time.
                         Diagram._add_morphism_closure(
-                            conclusions, morphism, FiniteSet(props),
+                            conclusions, morphism, FiniteSet(*props) if iterable(props) else FiniteSet(props),
                             add_identities=False, recurse_composites=False)
 
         return Basic.__new__(cls, Dict(premises), Dict(conclusions), objects)
@@ -740,6 +748,7 @@ class Diagram(Basic):
 
         Examples
         ========
+
         >>> from sympy.categories import Object, NamedMorphism
         >>> from sympy.categories import IdentityMorphism, Diagram
         >>> from sympy import pretty
@@ -749,7 +758,7 @@ class Diagram(Basic):
         >>> id_A = IdentityMorphism(A)
         >>> id_B = IdentityMorphism(B)
         >>> d = Diagram([f])
-        >>> print pretty(d.premises, use_unicode=False)
+        >>> print(pretty(d.premises, use_unicode=False))
         {id:A-->A: EmptySet(), id:B-->B: EmptySet(), f:A-->B: EmptySet()}
 
         """
@@ -762,6 +771,7 @@ class Diagram(Basic):
 
         Examples
         ========
+
         >>> from sympy.categories import Object, NamedMorphism
         >>> from sympy.categories import IdentityMorphism, Diagram
         >>> from sympy import FiniteSet
@@ -790,6 +800,7 @@ class Diagram(Basic):
 
         Examples
         ========
+
         >>> from sympy.categories import Object, NamedMorphism, Diagram
         >>> A = Object("A")
         >>> B = Object("B")
@@ -820,7 +831,7 @@ class Diagram(Basic):
         >>> f = NamedMorphism(A, B, "f")
         >>> g = NamedMorphism(B, C, "g")
         >>> d = Diagram([f, g], {g * f: "unique"})
-        >>> print pretty(d.hom(A, C), use_unicode=False)
+        >>> print(pretty(d.hom(A, C), use_unicode=False))
         ({g*f:A-->C}, {g*f:A-->C})
 
         See Also
@@ -850,6 +861,7 @@ class Diagram(Basic):
 
         Examples
         ========
+
         >>> from sympy.categories import Object, NamedMorphism, Diagram
         >>> A = Object("A")
         >>> B = Object("B")
@@ -863,14 +875,14 @@ class Diagram(Basic):
         >>> d1.is_subdiagram(d)
         False
         """
-        premises = all([(m in self.premises) and \
-                        (diagram.premises[m] == self.premises[m]) \
+        premises = all([(m in self.premises) and
+                        (diagram.premises[m] == self.premises[m])
                         for m in diagram.premises])
         if not premises:
             return False
 
-        conclusions = all([(m in self.conclusions) and \
-                           (diagram.conclusions[m] == self.conclusions[m]) \
+        conclusions = all([(m in self.conclusions) and
+                           (diagram.conclusions[m] == self.conclusions[m])
                         for m in diagram.conclusions])
 
         # Premises is surely ``True`` here.
@@ -885,6 +897,7 @@ class Diagram(Basic):
 
         Examples
         ========
+
         >>> from sympy.categories import Object, NamedMorphism, Diagram
         >>> from sympy import FiniteSet
         >>> A = Object("A")
@@ -897,17 +910,20 @@ class Diagram(Basic):
         >>> d1 == Diagram([f], {f: "unique"})
         True
         """
-        if not self.objects.subset(objects):
-            raise ValueError("Supplied objects should all belong to the diagram.")
+        if not objects.is_subset(self.objects):
+            raise ValueError(
+                "Supplied objects should all belong to the diagram.")
 
         new_premises = {}
         for morphism, props in self.premises.items():
-            if (morphism.domain in objects) and (morphism.codomain in objects):
+            if ((sympify(objects.contains(morphism.domain)) is S.true) and
+                (sympify(objects.contains(morphism.codomain)) is S.true)):
                 new_premises[morphism] = props
 
         new_conclusions = {}
         for morphism, props in self.conclusions.items():
-            if (morphism.domain in objects) and (morphism.codomain in objects):
+            if ((sympify(objects.contains(morphism.domain)) is S.true) and
+                (sympify(objects.contains(morphism.codomain)) is S.true)):
                 new_conclusions[morphism] = props
 
         return Diagram(new_premises, new_conclusions)
