@@ -1,13 +1,12 @@
-from sympy.functions import sqrt, sign, root
-from sympy.core import S, Wild, sympify, Mul, Add, Expr
-from sympy.core.function import expand_multinomial, expand_mul
-from sympy.core.symbol import Dummy
-from sympy.polys import Poly, PolynomialError
-from sympy.core.function import count_ops
-from sympy.utilities import default_sort_key
+from __future__ import print_function, division
 
-def _mexpand(expr):
-    return expand_mul(expand_multinomial(expr))
+from sympy.core import S, sympify, Mul, Add, Expr
+from sympy.core.compatibility import range
+from sympy.core.function import expand_mul, count_ops, _mexpand
+from sympy.core.symbol import Dummy
+from sympy.functions import sqrt, sign, root
+from sympy.polys import Poly, PolynomialError
+from sympy.utilities import default_sort_key
 
 
 def is_sqrt(expr):
@@ -38,7 +37,7 @@ def sqrt_depth(p):
     if p.is_Atom:
         return 0
     elif p.is_Add or p.is_Mul:
-        return max([sqrt_depth(x) for x in p.args])
+        return max([sqrt_depth(x) for x in p.args], key=default_sort_key)
     elif is_sqrt(p):
         return sqrt_depth(p.base) + 1
     else:
@@ -51,6 +50,7 @@ def is_algebraic(p):
 
     Examples
     ========
+
     >>> from sympy.functions.elementary.miscellaneous import sqrt
     >>> from sympy.simplify.sqrtdenest import is_algebraic
     >>> from sympy import cos
@@ -72,7 +72,7 @@ def is_algebraic(p):
         return False
 
 
-def subsets(n):
+def _subsets(n):
     """
     Returns all possible subsets of the set (0, 1, ..., n-1) except the
     empty set, listed in reversed lexicographical order according to binary
@@ -81,8 +81,8 @@ def subsets(n):
     Examples
     ========
 
-    >>> from sympy.simplify.sqrtdenest import subsets
-    >>> subsets(2)
+    >>> from sympy.simplify.sqrtdenest import _subsets
+    >>> _subsets(2)
     [[1, 0], [0, 1], [1, 1]]
 
     """
@@ -94,10 +94,10 @@ def subsets(n):
         a = [[1, 0, 0], [0, 1, 0], [1, 1, 0],
              [0, 0, 1], [1, 0, 1], [0, 1, 1], [1, 1, 1]]
     else:
-        b = subsets(n-1)
-        a0 = [x+[0] for x in b]
-        a1 = [x+[1] for x in b]
-        a = a0 + [[0]*(n-1) + [1]] + a1
+        b = _subsets(n - 1)
+        a0 = [x + [0] for x in b]
+        a1 = [x + [1] for x in b]
+        a = a0 + [[0]*(n - 1) + [1]] + a1
     return a
 
 
@@ -116,14 +116,16 @@ def sqrtdenest(expr, max_iter=3):
 
     See Also
     ========
+
     sympy.solvers.solvers.unrad
 
     References
     ==========
-    [1] http://www.almaden.ibm.com/cs/people/fagin/symb85.pdf
 
-    [2] D. J. Jeffrey and A. D. Rich, 'Symplifying Square Roots of Square Roots
-    by Denesting' (available at http://www.cybertester.com/data/denest.pdf)
+    .. [1] http://researcher.watson.ibm.com/researcher/files/us-fagin/symb85.pdf
+
+    .. [2] D. J. Jeffrey and A. D. Rich, 'Symplifying Square Roots of Square Roots
+           by Denesting' (available at http://www.cybertester.com/data/denest.pdf)
 
     """
     expr = expand_mul(sympify(expr))
@@ -141,12 +143,13 @@ def _sqrt_match(p):
 
     Examples
     ========
+
     >>> from sympy.functions.elementary.miscellaneous import sqrt
     >>> from sympy.simplify.sqrtdenest import _sqrt_match
     >>> _sqrt_match(1 + sqrt(2) + sqrt(2)*sqrt(3) +  2*sqrt(1+sqrt(5)))
     [1 + sqrt(2) + sqrt(6), 2, 1 + sqrt(5)]
     """
-    from sympy.simplify.simplify import split_surds
+    from sympy.simplify.radsimp import split_surds
 
     p = _mexpand(p)
     if p.is_Number:
@@ -161,7 +164,7 @@ def _sqrt_match(p):
         # so when the max is selected, it will be the largest arg having a
         # given depth
         v = [(sqrt_depth(x), x, i) for i, x in enumerate(pargs)]
-        nmax = max(v)
+        nmax = max(v, key=default_sort_key)
         if nmax[0] == 0:
             res = []
         else:
@@ -202,7 +205,6 @@ def _sqrt_match(p):
                             a1.append(x[1])
             a = Add(*a1)
             b = Add(*b1)
-            #a = Add._from_args(pargs)
             res = (a, b, r**2)
     else:
         b, r = p.as_coeff_Mul()
@@ -212,15 +214,17 @@ def _sqrt_match(p):
             res = []
     return list(res)
 
+
 class SqrtdenestStopIteration(StopIteration):
     pass
 
+
 def _sqrtdenest0(expr):
     """Returns expr after denesting its arguments."""
 
     if is_sqrt(expr):
         n, d = expr.as_numer_denom()
-        if d is S.One: # n is a square root
+        if d is S.One:  # n is a square root
             if n.base.is_Add:
                 args = sorted(n.base.args, key=default_sort_key)
                 if len(args) > 2 and all((x**2).is_Integer for x in args):
@@ -233,12 +237,25 @@ def _sqrtdenest0(expr):
         else:
             n, d = [_sqrtdenest0(i) for i in (n, d)]
             return n/d
+
+    if isinstance(expr, Add):
+        cs = []
+        args = []
+        for arg in expr.args:
+            c, a = arg.as_coeff_Mul()
+            cs.append(c)
+            args.append(a)
+
+        if all(c.is_Rational for c in cs) and all(is_sqrt(arg) for arg in args):
+            return _sqrt_ratcomb(cs, args)
+
     if isinstance(expr, Expr):
         args = expr.args
         if args:
             return expr.func(*[_sqrtdenest0(a) for a in args])
     return expr
 
+
 def _sqrtdenest_rec(expr):
     """Helper that denests the square root of three or more surds.
 
@@ -253,6 +270,7 @@ def _sqrtdenest_rec(expr):
 
     Examples
     ========
+
     >>> from sympy import sqrt
     >>> from sympy.simplify.sqrtdenest import _sqrtdenest_rec
     >>> _sqrtdenest_rec(sqrt(-72*sqrt(2) + 158*sqrt(5) + 498))
@@ -261,7 +279,7 @@ def _sqrtdenest_rec(expr):
     >>> _sqrtdenest_rec(sqrt(w))
     -sqrt(11) - sqrt(7) + sqrt(2) + 3*sqrt(5)
     """
-    from sympy.simplify.simplify import radsimp, split_surds, rad_rationalize
+    from sympy.simplify.radsimp import radsimp, rad_rationalize, split_surds
     if not expr.is_Pow:
         return sqrtdenest(expr)
     if expr.base < 0:
@@ -298,6 +316,7 @@ def _sqrtdenest_rec(expr):
     r = radsimp(r)
     return _mexpand(r)
 
+
 def _sqrtdenest1(expr, denester=True):
     """Return denested expr after denesting with simpler methods or, that
     failing, using the denester."""
@@ -369,13 +388,14 @@ def _sqrt_symbolic_denest(a, b, r):
 
     Examples
     ========
+
     >>> from sympy.simplify.sqrtdenest import _sqrt_symbolic_denest, sqrtdenest
     >>> from sympy import sqrt, Symbol
     >>> from sympy.abc import x
 
     >>> a, b, r = 16 - 2*sqrt(29), 2, -10*sqrt(29) + 55
     >>> _sqrt_symbolic_denest(a, b, r)
-    sqrt(-2*sqrt(29) + 11) + sqrt(5)
+    sqrt(11 - 2*sqrt(29)) + sqrt(5)
 
     If the expression is numeric, it will be simplified:
 
@@ -396,7 +416,7 @@ def _sqrt_symbolic_denest(a, b, r):
     sqrt(sqrt(sqrt(x + 3) + 1) + 1) + 1 + sqrt(2)
     """
 
-    a, b, r = sympify([a, b, r])
+    a, b, r = map(sympify, (a, b, r))
     rval = _sqrt_match(r)
     if not rval:
         return None
@@ -433,6 +453,7 @@ def _sqrt_numeric_denest(a, b, r, d2):
         vad1 = radsimp(1/vad)
         return (sqrt(vad/2) + sign(b)*sqrt((b**2*r*vad1/2).expand())).expand()
 
+
 def sqrt_biquadratic_denest(expr, a, b, r, d2):
     """denest expr = sqrt(a + b*sqrt(r))
     where a, b, r are linear combinations of square roots of
@@ -470,6 +491,7 @@ def sqrt_biquadratic_denest(expr, a, b, r, d2):
 
     Examples
     ========
+
     >>> from sympy import sqrt
     >>> from sympy.simplify.sqrtdenest import _sqrt_match, sqrt_biquadratic_denest
     >>> z = sqrt((2*sqrt(2) + 4)*sqrt(2 + sqrt(2)) + 5*sqrt(2) + 8)
@@ -478,7 +500,7 @@ def sqrt_biquadratic_denest(expr, a, b, r, d2):
     >>> sqrt_biquadratic_denest(z, a, b, r, d2)
     sqrt(2) + sqrt(sqrt(2) + 2) + 2
     """
-    from sympy.simplify.simplify import radsimp, rad_rationalize
+    from sympy.simplify.radsimp import radsimp, rad_rationalize
     if r <= 0 or d2 < 0 or not b or sqrt_depth(expr.base) < 2:
         return None
     for x in (a, b, r):
@@ -503,6 +525,7 @@ def sqrt_biquadratic_denest(expr, a, b, r, d2):
         return _mexpand(z)
     return None
 
+
 def _denester(nested, av0, h, max_depth_level):
     """Denests a list of expressions that contain nested square roots.
 
@@ -529,14 +552,14 @@ def _denester(nested, av0, h, max_depth_level):
     if av0[1] is None:
         return None, None
     if (av0[0] is None and
-        all(n.is_Number for n in nested)): # no arguments are nested
-        for f in subsets(len(nested)): # test subset 'f' of nested
+            all(n.is_Number for n in nested)):  # no arguments are nested
+        for f in _subsets(len(nested)):  # test subset 'f' of nested
             p = _mexpand(Mul(*[nested[i] for i in range(len(f)) if f[i]]))
             if f.count(1) > 1 and f[-1]:
                 p = -p
             sqp = sqrt(p)
             if sqp.is_Rational:
-                return sqp, f # got a perfect square so return its square root.
+                return sqp, f  # got a perfect square so return its square root.
         # Otherwise, return the radicand from the previous invocation.
         return sqrt(nested[-1]), [0]*len(nested)
     else:
@@ -547,9 +570,9 @@ def _denester(nested, av0, h, max_depth_level):
             nested2 = [av0[3], R]
             av0[0] = None
         else:
-            values = filter(None, [_sqrt_match(expr) for expr in nested])
+            values = list(filter(None, [_sqrt_match(expr) for expr in nested]))
             for v in values:
-                if v[2]: #Since if b=0, r is not defined
+                if v[2]:  # Since if b=0, r is not defined
                     if R is not None:
                         if R != v[2]:
                             av0[1] = None
@@ -562,7 +585,6 @@ def _denester(nested, av0, h, max_depth_level):
             nested2 = [_mexpand(v[0]**2) -
                        _mexpand(R*v[1]**2) for v in values] + [R]
         d, f = _denester(nested2, av0, h + 1, max_depth_level)
-        #d = sqrtdenest(d)
         if not f:
             return None, None
         if not any(f[i] for i in range(len(nested))):
@@ -574,7 +596,7 @@ def _denester(nested, av0, h, max_depth_level):
             if 1 in f and f.index(1) < len(nested) - 1 and f[len(nested) - 1]:
                 v[0] = -v[0]
                 v[1] = -v[1]
-            if not f[len(nested)]: #Solution denests with square roots
+            if not f[len(nested)]:  # Solution denests with square roots
                 vad = _mexpand(v[0] + d)
                 if vad <= 0:
                     # return the radicand from the previous invocation.
@@ -593,9 +615,53 @@ def _denester(nested, av0, h, max_depth_level):
                 return res, f
 
                       #          sign(v[1])*sqrt(_mexpand(v[1]**2*R*vad1/2))), f
-            else: #Solution requires a fourth root
+            else:  # Solution requires a fourth root
                 s2 = _mexpand(v[1]*R) + d
                 if s2 <= 0:
                     return sqrt(nested[-1]), [0]*len(nested)
                 FR, s = root(_mexpand(R), 4), sqrt(s2)
                 return _mexpand(s/(sqrt(2)*FR) + v[0]*FR/(sqrt(2)*s)), f
+
+
+def _sqrt_ratcomb(cs, args):
+    """Denest rational combinations of radicals.
+
+    Based on section 5 of [1].
+
+    Examples
+    ========
+
+    >>> from sympy import sqrt
+    >>> from sympy.simplify.sqrtdenest import sqrtdenest
+    >>> z = sqrt(1+sqrt(3)) + sqrt(3+3*sqrt(3)) - sqrt(10+6*sqrt(3))
+    >>> sqrtdenest(z)
+    0
+    """
+    from sympy.simplify.radsimp import radsimp
+
+    # check if there exists a pair of sqrt that can be denested
+    def find(a):
+        n = len(a)
+        for i in range(n - 1):
+            for j in range(i + 1, n):
+                s1 = a[i].base
+                s2 = a[j].base
+                p = _mexpand(s1 * s2)
+                s = sqrtdenest(sqrt(p))
+                if s != sqrt(p):
+                    return s, i, j
+
+    indices = find(args)
+    if indices is None:
+        return Add(*[c * arg for c, arg in zip(cs, args)])
+
+    s, i1, i2 = indices
+
+    c2 = cs.pop(i2)
+    args.pop(i2)
+    a1 = args[i1]
+
+    # replace a2 by s/a1
+    cs[i1] += radsimp(c2 * s / a1.base)
+
+    return _sqrt_ratcomb(cs, args)
